/*
 * Adaptor.h
 *
 *  Created on: 02.10.2012
 *      Author: Felix Ospald
 */

#ifndef FMG_ADAPTOR_H_
#define FMG_ADAPTOR_H_

#include <dolfin/fem/LinearVariationalProblem.h>
#include <dolfin/fem/NonlinearVariationalProblem.h>
#include <dolfin/mesh/Mesh.h>
#include <dolfin/mesh/MeshFunction.h>

namespace fmg {

/// Common methods for problem/mesh refinement based on *dolfin::adapt* with some minor
/// modifications for performance improvement.

class Adaptor
{
public:
	/// Refine mesh uniformly.
	static const dolfin::Mesh& adapt(const dolfin::Mesh& mesh);

	/// Refine mesh based on cell markers.
	static const dolfin::Mesh& adapt(const dolfin::Mesh& mesh, const dolfin::MeshFunction<bool>& cell_markers);

	/// Refine linear variational problem based on mesh.
	static const dolfin::LinearVariationalProblem& adapt(const dolfin::LinearVariationalProblem& problem,
            const dolfin::Mesh& adapted_mesh, bool interpolate_coef = false, bool interpolate_solution = false);

	/// Refine linear variational problem based on mesh (shared pointer).
	static const dolfin::LinearVariationalProblem& adapt(const dolfin::LinearVariationalProblem& problem,
            boost::shared_ptr<const dolfin::Mesh> adapted_mesh, bool interpolate_coef = false, bool interpolate_solution = false);

	/// Refine nonlinear variational problem based on mesh.
	static const dolfin::NonlinearVariationalProblem& adapt(const dolfin::NonlinearVariationalProblem& problem,
            const dolfin::Mesh& adapted_mesh, bool interpolate_coef = false, bool interpolate_solution = false);

	/// Refine nonlinear variational problem based on mesh (shared pointer).
	static const dolfin::NonlinearVariationalProblem& adapt(const dolfin::NonlinearVariationalProblem& problem,
            boost::shared_ptr<const dolfin::Mesh> adapted_mesh, bool interpolate_coef = false, bool interpolate_solution = false);
};

} // namespace

#endif /* FMG_ADAPTOR_H_ */
