/*
 * Compat.h
 *
 *  Created on: 25.10.2012
 *      Author: Felix Ospald
 */

/// This file contains some definitions to provide compatibility between Dolfin 1.0.0 and 1.0.0+.

#ifndef FMG_COMPAT_H_
#define FMG_COMPAT_H_

#include <dolfin/la/GenericMatrix.h>

#ifdef __GENERIC_LINEAR_OPERATOR_H
	#define FMG_DOLFIN_VERSION 2
#else
	#define FMG_DOLFIN_VERSION 1
#endif

#if FMG_DOLFIN_VERSION > 1
	#define FMG_FACTORY_VECTOR(f) (f).create_vector()
	#define FMG_FACTORY_MATRIX(f) (f).create_matrix()
	#define FMG_FACTORY_KRYLOV_SOLVER(f,m,p) (f).create_krylov_solver(m,p)
	#define FMG_FACTORY_LU_SOLVER(f,m) (f).create_lu_solver(m)
	#define FMG_COPY_VECTOR(v) (v).copy()
#else
	#ifndef GenericLinearAlgebraFactory
		#define GenericLinearAlgebraFactory LinearAlgebraFactory
	#endif
	#ifndef GenericLinearOperator
		#define GenericLinearOperator GenericMatrix
	#endif
	#define FMG_FACTORY_VECTOR(f) boost::shared_ptr<dolfin::GenericVector>((f).create_vector())
	#define FMG_FACTORY_MATRIX(f) boost::shared_ptr<dolfin::GenericMatrix>((f).create_matrix())
	#define FMG_FACTORY_KRYLOV_SOLVER(f,m,p) boost::shared_ptr<dolfin::GenericLinearSolver>((f).create_krylov_solver(m,p))
	#define FMG_FACTORY_LU_SOLVER(f,m) boost::shared_ptr<dolfin::GenericLinearSolver>((f).create_lu_solver(m))
	#define FMG_COPY_VECTOR(v) boost::shared_ptr<dolfin::GenericVector>((v).copy())
#endif

#define FMG_NOT_IMPLEMENTED() dolfin::error("perform call to FMG function", "The function %s has not been implemented (in %s line %d)", __FUNCTION__, __FILE__, __LINE__)

#endif /* FMG_COMPAT_H_ */
