/*
 * FMGTimer.h.cpp
 *
 *  Created on: 01.10.2012
 *      Author: Felix Ospald
 */

#include "FMGTimer.h"

namespace fmg {

// Static variable for storing the tasks.
std::map<std::string, FMGTimer::task_t> FMGTimer::_tasks;


// Stop the timer and update statistics.
double FMGTimer::stop()
{
  double dt = dolfin::Timer::stop();
  task_t& task = _tasks[_task];
  task.n++;
  if (task.n == 1) {
	  task.t_sum = task.t_min = task.t_max = task.t_mean = dt;
  }
  else {
	  task.t_sum += dt;
	  task.t_min = std::min(task.t_min, dt);
	  task.t_max = std::max(task.t_max, dt);
	  task.t_mean = task.t_sum/task.n;
  }
  _stopped = true;
  return dt;
}


// Get a task by name.
FMGTimer::task_t& FMGTimer::get_task(const std::string& task)
{
	if (!have_task(task)) {
		dolfin::error("timer task '%s' not found!", task.c_str());
	}

	return _tasks[task];
}

} // namespace
