/*
 * SmoothingOperator.h
 *
 *  Created on: 02.10.2012
 *      Author: Felix Ospald
 */

#ifndef FMG_GENERICSMOOTHINGOPERATOR_H_
#define FMG_GENERICSMOOTHINGOPERATOR_H_

#include <dolfin/la/GenericVector.h>

namespace fmg {

/// This class provides a generic interface for using various smoothing operators.

class GenericSmoothingOperator
{
public:
	/// Applies the smoothing operator to *x* for given rhs *b* of the problem *reps* times.
	/// If *x_assumed_zero* is *true then the computed result is as *x* where zero.
	/// In general *x_assumed_zero* is set to false if *reps* > 0 before return of the function.
	virtual void apply(dolfin::GenericVector& x, const dolfin::GenericVector& b, bool& x_assumed_zero, std::size_t reps) const = 0;
};

} // namespace


#endif /* FMG_GENERICSMOOTHINGOPERATOR_H_ */
