/*
 * JacobiSmoothingOperator.h
 *
 *  Created on: 02.10.2012
 *      Author: Felix Ospald
 */

#ifndef FMG_JACOBISMOOTHINGOPERATOR_H_
#define FMG_JACOBISMOOTHINGOPERATOR_H_

#include <fmg/relax/GenericSmoothingOperator.h>
#include <dolfin/la/GenericMatrix.h>

namespace fmg {

/// Implementation of Jacobi relaxation sweep:
/// :math:`x_{k+1} = x_k - \omega (\mathrm{diag} A)^{-1} (A x_k - b)`

class JacobiSmoothingOperator : public GenericSmoothingOperator
{
public:
	/// Create Jacobi smoother for matrix *A* and relaxation parameter *omega*.
	JacobiSmoothingOperator(const boost::shared_ptr<const dolfin::GenericMatrix>& A, double omega = 1.0);

	void apply(dolfin::GenericVector& x, const dolfin::GenericVector& b, bool& x_assumed_zero, std::size_t reps) const;

protected:
	boost::shared_ptr<const dolfin::GenericMatrix> _A;
	boost::shared_ptr<dolfin::GenericVector> _diag;
	boost::shared_ptr<dolfin::GenericVector> _y;
};

} // namespace


#endif /* FMG_PROLONGATIONOPERATOR_H_ */
