/*
 * MultigridPreconditioner.cpp
 *
 *  Created on: 01.10.2012
 *      Author: Felix Ospald
 */

#include "MultigridPreconditioner.h"
#include "../common/Utils.h"

#include <petscversion.h>
#if PETSC_VERSION_MAJOR==3 && PETSC_VERSION_MINOR>2
	#include <petsc-private/pcimpl.h>
#else
	#include <private/pcimpl.h>
#endif

namespace fmg {

MultigridPreconditioner::MultigridPreconditioner(MultigridSolver& mg_solver)
{
	init(dolfin::reference_to_no_delete_pointer(mg_solver));
}


MultigridPreconditioner::MultigridPreconditioner(const boost::shared_ptr<MultigridSolver>& mg_solver)
{
	init(mg_solver);
}


void MultigridPreconditioner::init(const boost::shared_ptr<MultigridSolver>& mg_solver)
{
	_mg_solver = mg_solver;

	// setup solver parameters used for preconditioning
	mg_solver->parameters["error_on_nonconvergence"] = false;
	mg_solver->parameters["tolerance_checking"] = false;
	mg_solver->parameters["maximum_iterations"] = 1;
}


void MultigridPreconditioner::setup(dolfin::PETScKrylovSolver& solver)
{
	dolfin::PETScUserPreconditioner::setup(*(solver.ksp()), *this);

#if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR > 1
	PC petscpc;
	KSPGetPC(*(solver.ksp()), &petscpc);
	petscpc->ops->reset = 0;
#endif

	if (solver.parameters.has_parameter("convergence_norm_type")) {
		solver.parameters["convergence_norm_type"] = "preconditioned";
	}
	KSPSetNormType(*(solver.ksp()), KSP_NORM_PRECONDITIONED);
}


void MultigridPreconditioner::solve(dolfin::PETScVector& x, const dolfin::PETScVector& b)
{
	//fmg::Utils::print_vector("b", b);
	//std::cout << "bnorm: " << b.norm("linf") << std::endl;

	// x = P*b
#if 1
	_mg_solver->solve(x, b, true);
#else
	x.zero();
	_mg_solver->solve(x, b, false);
#endif

	//fmg::Utils::print_vector("x", x);
	//std::cout << "xnorm: " << x.norm("linf") << std::endl;
	//std::cout << "def: " << b.inner(x) << std::endl;
}

} // namespace

